<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

use App\Management;
use App\Client;
use App\User;

class ReportRecord extends Model
{

    const STATUS_CHANGED = 10;
    const MANAGEMENT_ASSIGN = 11;
    const USER_ASSIGN = 12;
    const MANAGEMENT_UNASSIGN = 13;
    const USER_UNASSIGN = 14;
    const CONTACT_ATTEMPT = 16;
    const CREATED_EVENT = 21;
    const RESCHEDULED_EVENT = 22;
    const CANCELLED_EVENT = 23;
    const DONE_EVENT = 24;
    const LOCATION_SELECTED_EVENT = 25;
    const RELOCATED_EVENT = 26;
    const ADDON_SERVICE = 31;
    const REGULAR_SERVICE = 32;
    const USER_DELETED = 40;
    const USER_RESTORED = 41;

    protected $fillable = [
        'kind'
    ];

    protected $appends = ['action', 'column'];

    public function management()
    {
        return $this->belongsTo('App\Management');
    }

    public function client()
    {
        return $this->belongsTo('App\Client')->withTrashed();
    }

    public function user()
    {
        return $this->belongsTo('App\User')->withTrashed();
    }

    public function oldUser()
    {
        return $this->belongsTo('App\User', 'start_value')->withTrashed();
    }

    public function newUser()
    {
        return $this->belongsTo('App\User', 'end_value')->withTrashed();
    }

    public function oldManagement()
    {
        return $this->belongsTo('App\Management', 'start_value');
    }

    public function newManagement()
    {
        return $this->belongsTo('App\Management', 'end_value');
    }

    public function executive()
    {
        return $this->belongsTo('App\User');
    }

    public function event()
    {
        return $this->belongsTo('App\Event')->withTrashed();
    }

    public function requiredService()
    {
        return $this->belongsTo('App\RequiredService');
    }

    public function getActionAttribute()
    {
        switch ($this->kind) {
            case 10:
                return "Estatus actualizado";
            case 11:
                return 'Asignación de gerencia';
            case 12:
                return 'Asignación de ejecutivo';
            case 13:
                return 'Gerencia desasignada';
            case 14:
                return 'Usuario desasignado';
            case self::CONTACT_ATTEMPT : 
                return 'Intento de contacto';
            case 21:
                if(($event = $this->event) && $event->kind != 'visit') {
                    return 'Compromiso agendado';
                }
                return 'Cita agendada';
            case 22:
                if(($event = $this->event) && $event->kind != 'visit') {
                    return 'Compromiso reagendado';
                }
                return 'Cita reagendada';
            case 23:
                return 'Cita cancelada';
            case 24:
                if(($event = $this->event) && $event->kind != 'visit') {
                    return 'Compromiso concretado';
                }
                return 'Cita concretada';
            case 25:
                return 'Ubicación para cita seleccionada';
            case 26:
                return 'Ubicación actualizada';
            case 31:
                return 'Servicio adicional requerido';                    
            case 32:
                return 'Servicio requerido';
            case self::USER_RESTORED :
                return 'Cliente restaurado';
            case self::USER_DELETED :
                $reason = $this->end_value;
                switch ($reason) {
                    case Client::ESTATUS_CEDIDO:
                        return 'Cliente cedido';
                    case Client::ESTATUS_FINADO:
                        return 'Cliente finado';
                    case Client::ESTATUS_PENSIONADO:
                        return 'Cliente pensionado';
                }
                return 'Cliente cedido';
            default: \Log::debug($this->kind); return 'N/A';
        }
    }

    public function getDescriptionAttribute()
    {
         switch ($this->kind) {
            case 10 :
                return \Lang::get('clients.status.'.$this->end_value);
            case 11:

                return "Gerencia Asignada : ".($this->newManagement ? $this->newManagement->name : "Gerencia dada de baja");

            case 12:
                return "Ejecutivo Asignado : ". ($this->newUser ? $this->newUser->full_name : "Dado de baja");
            case 13:
                return 'Gerencia previa: '. ($this->oldManagement ? $this->oldManagement->name : "GERENCIA DADA DE BAJA");
            case 14:
                $u =  $this->oldUser;
                if($u){
                    return 'Ejecutivo anterior: ' .$u->full_name;        
                }else{
                    return 'Ejecutivo anterior: EJECUTIVO DADA DE BAJA';        
                }
            case self::CONTACT_ATTEMPT :
                return "NO CONTACTADO";
            case 21:
                $date = 'N/A';
                try {
                    $date = \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $this->end_value)->format('d/m/Y H:i');    
                } catch (Exception $e) {
                    $date = 'N/A';
                }finally{
                    return $date;    
                }

                

            case 22:
                $date = 'N/A';
                try {
                    $date = \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $this->end_value)->format('d/m/Y H:i');
                 } catch (Exception $e) {
                     $date = 'N/A';
                 } finally {
                    return $date;    
                 } 
                
            case 23: 
                return 'Cita cancelada';
            case 24:
                $result ='';
                if($service = $this->event->requiredService){
                    if($service->service == 'addon') {
                        $result = $service->description;
                    }else {
                        $result = \Lang::get("clients.services.{$service->service}");    
                    }
                    
                }
                return $result;
            case 25:
                return 'Ubicación para cita seleccionada';
            case 26:
                return 'Ubicación actualizada';
            case 31:
                return $this->requiredService->description;
                           
            case 32:
                if($service = $this->requiredService){
                    $result = \Lang::get("clients.services.{$service->service}");
                }

                return $result;

            default: 
                return '';
        }
    }

    public function getCommentsAttribute()
    {
        switch ($this->kind) {
            case self::ADDON_SERVICE:
            case self::REGULAR_SERVICE :
                $result = '';
                if ($service = $this->requiredService) {
                    $result = 'Documentación';
                    if($service->status == 0) 
                        $result = ' No entregada';
                    elseif($service->status == 1) $result = $result . ' Entregada';
                    elseif($service->status == 2) $result = $result . ' Pendiente';
                }
                return $result;
            case self::CREATED_EVENT :
                return $this->event->title;
            case self::DONE_EVENT :
                return $this->event->title.'. '.$this->event->comments;
            case self::STATUS_CHANGED :
                switch ($this->end_value) {
                    case Client::ESTATUS_CITA_RECHAZADA:
                        return $this->client->status_description;
                    
                }
                return '';
            case self::CONTACT_ATTEMPT :
                return $this->end_value;
            default :
                return '';
        }

    }

    public function getColumnAttribute()
    {
        return @$this->attributes['column_'];
    }

    public function setColumnAttribute($value)
    {
        $this->attributes['column_'] = $value;
    }
}
